@@ -1,18 +1,14 @@
-/* Copyright (c) 2023 The Khronos Group Inc.
+/* Copyright (c) 2023-2025 The Khronos Group Inc.
  * SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: Apache-2.0
+ * SPDX-License-Identifier: Apache-2.0 or MIT
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed under either of
+ *   Apache License, Version 2.0 (http://www.apache.org/licenses/LICENSE-2.0)
+ *   or
+ *   MIT license (http://opensource.org/licenses/MIT)
+ * at your option.
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Any contribution submitted by you to Khronos for inclusion in this work shall be dual licensed as above.
  *
  * Author: Vikram Kushwaha <vkushwaha@nvidia.com>
  */
@@ -205,7 +201,8 @@ VKAPI_ATTR VkResult VKAPI_CALL EnumerateDeviceExtensionProperties(VkPhysicalDevi
     return vk_outarray_status(&out);
 }
 
-VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceProperties2(VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties) {
+void GetPhysicalDeviceProperties2Internal(VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties) {
+    /* query properties from the underlying Vulkan implementation */
     auto instance_data = GetInstanceData(physicalDevice);
     auto pdd = instance_data->GetPhysicalDeviceData(physicalDevice);
 
@@ -214,7 +211,8 @@ VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceProperties2(VkPhysicalDevice physica
     }
 }
 
-VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceFeatures2(VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures) {
+void GetPhysicalDeviceFeatures2Internal(VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures) {
+    /* query features from the underlying Vulkan implementation */
     auto instance_data = GetInstanceData(physicalDevice);
     auto pdd = instance_data->GetPhysicalDeviceData(physicalDevice);
 
@@ -223,6 +221,50 @@ VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceFeatures2(VkPhysicalDevice physicalD
     }
 }
 
+VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceProperties2(VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties) {
+    GetPhysicalDeviceProperties2Internal(physicalDevice, pProperties);
+
+    auto subject = pProperties;
+    while(subject) {
+        if (subject->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_PROPERTIES_NV) {
+            auto as_decompressionProperties =
+                reinterpret_cast<VkPhysicalDeviceMemoryDecompressionPropertiesNV*>(subject);
+            if (as_decompressionProperties->decompressionMethods == 0) {
+                // no decompression methods provided by underlying implementation; layer wil use GDeflate
+                as_decompressionProperties->decompressionMethods = VK_MEMORY_DECOMPRESSION_METHOD_GDEFLATE_1_0_BIT_NV;
+                // 2^16-1, the minimum allowed by the Vulkan spec, version 1.4.314
+                as_decompressionProperties->maxDecompressionIndirectCount = 65535;
+            }
+            break;
+        }
+        subject = static_cast<VkPhysicalDeviceProperties2*>(subject->pNext);
+    }
+}
+
+VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceFeatures2(VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures) {
+    GetPhysicalDeviceFeatures2Internal(physicalDevice, pFeatures);
+
+    auto subject = pFeatures;
+    while(subject) {
+        if (subject->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_FEATURES_NV) {
+            auto as_decompressionFeatures =
+                reinterpret_cast<VkPhysicalDeviceMemoryDecompressionFeaturesNV*>(subject);
+            // we're guaranteed support; either the driver or layer supports this extension.
+            as_decompressionFeatures->memoryDecompression = VK_TRUE;
+            break;
+        }
+        subject = static_cast<VkPhysicalDeviceFeatures2*>(subject->pNext);
+    }
+}
+
+VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceProperties2KHR(VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties) {
+    GetPhysicalDeviceProperties2(physicalDevice, pProperties);
+}
+
+VKAPI_ATTR void VKAPI_CALL GetPhysicalDeviceFeatures2KHR(VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures) {
+    GetPhysicalDeviceFeatures2(physicalDevice, pFeatures);
+}
+
 #define INIT_HOOK(_vt, _inst, fn) _vt.fn = reinterpret_cast<PFN_vk##fn>(vtable.GetInstanceProcAddr(_inst, "vk" #fn))
 InstanceData::InstanceData(VkInstance inst, PFN_vkGetInstanceProcAddr gpa, const VkAllocationCallbacks* alloc)
     : instance(inst), api_version(0), allocator(alloc) {
@@ -235,6 +277,8 @@ InstanceData::InstanceData(VkInstance inst, PFN_vkGetInstanceProcAddr gpa, const
     INIT_HOOK(vtable, instance, EnumerateInstanceExtensionProperties);
     INIT_HOOK(vtable, instance, GetPhysicalDeviceProperties2);
     INIT_HOOK(vtable, instance, GetPhysicalDeviceFeatures2);
+    INIT_HOOK(vtable, instance, GetPhysicalDeviceProperties2KHR);
+    INIT_HOOK(vtable, instance, GetPhysicalDeviceFeatures2KHR);
     INIT_HOOK(vtable, instance, GetPhysicalDeviceProperties);
     INIT_HOOK(vtable, instance, GetPhysicalDeviceMemoryProperties);
 }
@@ -260,18 +304,12 @@ void InitLayerSettings(const VkInstanceCreateInfo* pCreateInfo, const VkAllocati
     static const char* setting_names[] = {kLayerSettingsForceEnable, kLayerSettingsLogging, kLayerSettingsCustomSTypeInfo};
     uint32_t setting_name_count = static_cast<uint32_t>(std::size(setting_names));
 
-    uint32_t unknown_setting_count = 0;
-    vkuGetUnknownSettings(create_info, setting_name_count, setting_names, &unknown_setting_count, nullptr);
+    std::vector<const char*> unknown_settings;
 
-    if (unknown_setting_count > 0) {
-        std::vector<const char*> unknown_settings;
-        unknown_settings.resize(unknown_setting_count);
+    vkuGetUnknownSettings(layer_setting_set, setting_name_count, setting_names, create_info, unknown_settings);
 
-        vkuGetUnknownSettings(create_info, setting_name_count, setting_names, &unknown_setting_count, &unknown_settings[0]);
-
-        for (std::size_t i = 0, n = unknown_settings.size(); i < n; ++i) {
-            LOG("Unknown %s setting listed in VkLayerSettingsCreateInfoEXT, this setting is ignored.\n", unknown_settings[i]);
-        }
+    for (std::size_t i = 0, n = unknown_settings.size(); i < n; ++i) {
+        LOG("Unknown %s setting listed in VkLayerSettingsCreateInfoEXT, this setting is ignored.\n", unknown_settings[i]);
     }
 
     if (vkuHasLayerSetting(layer_setting_set, kLayerSettingsForceEnable)) {
@@ -307,7 +345,7 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateInstance(const VkInstanceCreateInfo* pCreat
     if (result != VK_SUCCESS) {
         return result;
     }
-    try {
+    {
         auto instance_data =
             std::make_shared<InstanceData>(*pInstance, gpa, pAllocator ? pAllocator : &extension_layer::kDefaultAllocator);
 
@@ -317,10 +355,6 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateInstance(const VkInstanceCreateInfo* pCreat
 
         InitLayerSettings(pCreateInfo, pAllocator, &instance_data->layer_settings);
         logging_enabled = instance_data->layer_settings.logging;
-    } catch (const std::bad_alloc&) {
-        auto destroy_instance = reinterpret_cast<PFN_vkDestroyInstance>(gpa(NULL, "vkDestroyInstance"));
-        destroy_instance(*pInstance, pAllocator);
-        result = VK_ERROR_OUT_OF_HOST_MEMORY;
     }
     return result;
 }
@@ -448,22 +482,38 @@ VkResult DeviceData::CreatePipelineState(VkDevice* pDevice, VkPhysicalDevice phy
     VkPhysicalDeviceSubgroupSizeControlProperties subgroupsizeProps = {
         VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_PROPERTIES};
     props.pNext = &subgroupsizeProps;
-    GetPhysicalDeviceProperties2(physicalDevice, &props);
+    GetPhysicalDeviceProperties2Internal(physicalDevice, &props);
 
     VkPhysicalDeviceFeatures2 devFeatures = {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2};
     VkPhysicalDeviceSubgroupSizeControlFeatures subgroupFeatures = {
         VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_FEATURES};
     devFeatures.pNext = &subgroupFeatures;
-    GetPhysicalDeviceFeatures2(physicalDevice, &devFeatures);
+    GetPhysicalDeviceFeatures2Internal(physicalDevice, &devFeatures);
 
     uint32_t subgroupSize = 32;
     if (!subgroupFeatures.subgroupSizeControl || subgroupsizeProps.minSubgroupSize < 16) {
         // If we do not have size control extension, or width < 16
         // Force a shader variant with full syncronization.
         subgroupSize = 8;
+
+        // Intel Xe (and earlier) GPUs can support subgroup sizes of 8, 16,
+        // and 32. At least on current versions of the Intel compiler in Mesa,
+        // the SIMD32 version cannot fit in available register space, and it
+        // is about 11% slower than the SIMD16 version.
+        if (subgroupsizeProps.minSubgroupSize == 8 && subgroupsizeProps.maxSubgroupSize == 32) {
+            subgroupSize = 16;
+        }
     } else if (subgroupFeatures.subgroupSizeControl) {
-        // Use a shader with the narrowest supported subgroup.
-        subgroupSize = subgroupsizeProps.minSubgroupSize;
+        // Intel Xe2 GPUs can support subgroup sizes of 16 and 32. The
+        // GInflate shader will easily fit in the register file of SIMD32, and
+        // there are significant advantages to the 32-wide shader. Force a
+        // subgroup size of 32.
+        if (subgroupsizeProps.maxSubgroupSize <= 32) {
+            subgroupSize = subgroupsizeProps.maxSubgroupSize;
+        } else {
+            // Use a shader with the narrowest supported subgroup.
+            subgroupSize = subgroupsizeProps.minSubgroupSize;
+        }
     }
 
     PRINT("Info: subgroupSize %u\n", subgroupSize);
@@ -558,6 +608,10 @@ VkResult DeviceData::CreatePipelineState(VkDevice* pDevice, VkPhysicalDevice phy
     VkShaderModule copyCountModule;
     VkShaderModule indirectDecompressShaderModule;
 
+    VkPipelineShaderStageRequiredSubgroupSizeCreateInfo rss_info = {
+        VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_REQUIRED_SUBGROUP_SIZE_CREATE_INFO};
+    rss_info.requiredSubgroupSize = subgroupSize;
+
     // Create Decompression shader pipeline
     ByteCode bytecode = kGInflateBytecode[bytecodeIndex];
     VkShaderModuleCreateInfo shaderModuleInfo = {VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO};
@@ -586,6 +640,11 @@ VkResult DeviceData::CreatePipelineState(VkDevice* pDevice, VkPhysicalDevice phy
     pipelineInfo.stage.stage = VK_SHADER_STAGE_COMPUTE_BIT;
     pipelineInfo.stage.module = decompressShaderModule;
     pipelineInfo.stage.pName = "main";
+
+    if (subgroupFeatures.subgroupSizeControl) {
+        pipelineInfo.stage.pNext = &rss_info;
+    }
+
     result = vtable.CreateComputePipelines(device, VK_NULL_HANDLE, 1, &pipelineInfo, 0, &pipelineDecompressSingle);
     if (result != VK_SUCCESS) {
         return result;
@@ -614,6 +673,7 @@ VkResult DeviceData::CreatePipelineState(VkDevice* pDevice, VkPhysicalDevice phy
     pipelineInfo.stage.stage = VK_SHADER_STAGE_COMPUTE_BIT;
     pipelineInfo.stage.module = copyCountModule;
     pipelineInfo.stage.pName = "main";
+    pipelineInfo.stage.pNext = NULL;
     result = vtable.CreateComputePipelines(device, VK_NULL_HANDLE, 1, &pipelineInfo, 0, &pipelineCopy);
     if (result != VK_SUCCESS) {
         return result;
@@ -643,6 +703,11 @@ VkResult DeviceData::CreatePipelineState(VkDevice* pDevice, VkPhysicalDevice phy
     pipelineInfo.stage.stage = VK_SHADER_STAGE_COMPUTE_BIT;
     pipelineInfo.stage.module = indirectDecompressShaderModule;
     pipelineInfo.stage.pName = "main";
+
+    if (subgroupFeatures.subgroupSizeControl) {
+        pipelineInfo.stage.pNext = &rss_info;
+    }
+
     result = vtable.CreateComputePipelines(device, VK_NULL_HANDLE, 1, &pipelineInfo, 0, &pipelineDecompressMulti);
     if (result != VK_SUCCESS) {
         return result;
@@ -681,17 +746,17 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateDevice(VkPhysicalDevice physicalDevice, con
     VkPhysicalDeviceVulkan12Features vulkan12Features = {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES};
     decompressionFeature.pNext = &vulkan12Features;
     devFeatures.pNext = &decompressionFeature;
-    GetPhysicalDeviceFeatures2(physicalDevice, &devFeatures);
+    GetPhysicalDeviceFeatures2Internal(physicalDevice, &devFeatures);
 
     VkPhysicalDeviceProperties2 props = {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2};
     VkPhysicalDeviceSubgroupProperties subgroupProps = {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_PROPERTIES};
     props.pNext = &subgroupProps;
-    GetPhysicalDeviceProperties2(physicalDevice, &props);
+    GetPhysicalDeviceProperties2Internal(physicalDevice, &props);
 
     const bool computeStageSupport = (subgroupProps.supportedStages & VK_SHADER_STAGE_COMPUTE_BIT);
     const bool subgroupBasicSupport = subgroupProps.supportedOperations & VK_SUBGROUP_FEATURE_BASIC_BIT;
 
-    try {
+    {
         bool enable_layer = false;
         if (!decompressionFeature.memoryDecompression) {
             PRINT("Memory decompression feature not available in the driver, enabling decompression layer.\n");
@@ -746,10 +811,6 @@ VKAPI_ATTR VkResult VKAPI_CALL CreateDevice(VkPhysicalDevice physicalDevice, con
         }
 
         device_data_map.insert(DispatchKey(*pDevice), device_data);
-    } catch (const std::bad_alloc&) {
-        auto destroy_device = reinterpret_cast<PFN_vkDestroyDevice>(gdpa(*pDevice, "vkDestroyDevice"));
-        destroy_device(*pDevice, pAllocator);
-        result = VK_ERROR_OUT_OF_HOST_MEMORY;
     }
     return result;
 }
@@ -807,67 +868,54 @@ static VkPipelineStageFlags ConvertPipelineStageMask(VkPipelineStageFlags2KHR st
 
 VKAPI_ATTR void VKAPI_CALL CmdDecompressMemoryNV(VkCommandBuffer commandBuffer, uint32_t decompressRegionCount,
                                                  VkDecompressMemoryRegionNV const* pDecompressMemoryRegions) {
-    try {
-        auto device_data = GetDeviceData(commandBuffer);
+    auto device_data = GetDeviceData(commandBuffer);
 
-        if (device_data->vtable.CmdDecompressMemoryNV) {
-            device_data->vtable.CmdDecompressMemoryNV(commandBuffer, decompressRegionCount, pDecompressMemoryRegions);
-        } else {
-            device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE,
-                                                device_data->pipelineDecompressSingle);
-            for (uint32_t t = 0; t < decompressRegionCount; t++) {
-                device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutDecompressSingle,
-                                                     VK_SHADER_STAGE_COMPUTE_BIT, 0, sizeof(VkDecompressMemoryRegionNV),
-                                                     (void*)&pDecompressMemoryRegions[t]);
-                device_data->vtable.CmdDispatch(commandBuffer, 1, 1, 1);
-                PRINT("Info: vkCmdDecompressMemoryNV: Using VK_LAYER_KHRONOS_memory_decompression layer\n");
-            }
+    if (device_data->vtable.CmdDecompressMemoryNV) {
+        device_data->vtable.CmdDecompressMemoryNV(commandBuffer, decompressRegionCount, pDecompressMemoryRegions);
+    } else {
+        device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE, device_data->pipelineDecompressSingle);
+        for (uint32_t t = 0; t < decompressRegionCount; t++) {
+            device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutDecompressSingle,
+                                                 VK_SHADER_STAGE_COMPUTE_BIT, 0, sizeof(VkDecompressMemoryRegionNV),
+                                                 (void*)&pDecompressMemoryRegions[t]);
+            device_data->vtable.CmdDispatch(commandBuffer, 1, 1, 1);
+            PRINT("Info: vkCmdDecompressMemoryNV: Using VK_LAYER_KHRONOS_memory_decompression layer\n");
         }
-    } catch (const std::bad_alloc& e) {
-        // We don't have a way to return an error here.
-        PRINT("bad_alloc: %s\n", e.what());
     }
 }
 
 VKAPI_ATTR void VKAPI_CALL CmdDecompressMemoryIndirectCountNV(VkCommandBuffer commandBuffer,
                                                               VkDeviceAddress indirectCommandsAddress,
                                                               VkDeviceAddress indirectCommandsCountAddress, uint32_t stride) {
-    try {
-        auto device_data = GetDeviceData(commandBuffer);
-        if (device_data->vtable.CmdDecompressMemoryIndirectCountNV) {
-            device_data->vtable.CmdDecompressMemoryIndirectCountNV(commandBuffer, indirectCommandsAddress,
-                                                                   indirectCommandsCountAddress, stride);
-        } else {
-            DeviceData::PushConstantDataCopy pushConstantDataCopy = {indirectCommandsCountAddress,
-                                                                     device_data->indirectDispatchBufferAddress};
-            device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE, device_data->pipelineCopy);
-            device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutCopy, VK_SHADER_STAGE_COMPUTE_BIT, 0,
-                                                 sizeof(DeviceData::PushConstantDataCopy), &pushConstantDataCopy);
-            device_data->vtable.CmdDispatch(commandBuffer, 1, 1, 1);
+    auto device_data = GetDeviceData(commandBuffer);
+    if (device_data->vtable.CmdDecompressMemoryIndirectCountNV) {
+        device_data->vtable.CmdDecompressMemoryIndirectCountNV(commandBuffer, indirectCommandsAddress, indirectCommandsCountAddress,
+                                                               stride);
+    } else {
+        DeviceData::PushConstantDataCopy pushConstantDataCopy = {indirectCommandsCountAddress,
+                                                                 device_data->indirectDispatchBufferAddress};
+        device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE, device_data->pipelineCopy);
+        device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutCopy, VK_SHADER_STAGE_COMPUTE_BIT, 0,
+                                             sizeof(DeviceData::PushConstantDataCopy), &pushConstantDataCopy);
+        device_data->vtable.CmdDispatch(commandBuffer, 1, 1, 1);
 
-            DeviceData::PushConstantDataDecompressMulti pushConstantData = {indirectCommandsAddress, stride};
-            {
-                VkBufferMemoryBarrier bufferBarrier = {VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER};
-                bufferBarrier.buffer = device_data->indirectDispatchBuffer;
-                bufferBarrier.offset = 0;
-                bufferBarrier.size = VK_WHOLE_SIZE;
-                bufferBarrier.srcAccessMask = VK_ACCESS_SHADER_WRITE_BIT;
-                bufferBarrier.dstAccessMask = VK_ACCESS_INDIRECT_COMMAND_READ_BIT;
-                device_data->vtable.CmdPipelineBarrier(commandBuffer, VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT,
-                                                       VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, 0, 0, 1, &bufferBarrier, 0, 0);
-            }
-
-            device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE,
-                                                device_data->pipelineDecompressMulti);
-            device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutDecompressMulti,
-                                                 VK_SHADER_STAGE_COMPUTE_BIT, 0,
-                                                 sizeof(DeviceData::PushConstantDataDecompressMulti), &pushConstantData);
-            device_data->vtable.CmdDispatchIndirect(commandBuffer, device_data->indirectDispatchBuffer, 0);
-            PRINT("Info: vkCmdDecompressMemoryIndirectCountNV: Using VK_LAYER_KHRONOS_memory_decompression layer\n");
+        DeviceData::PushConstantDataDecompressMulti pushConstantData = {indirectCommandsAddress, stride};
+        {
+            VkBufferMemoryBarrier bufferBarrier = {VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER};
+            bufferBarrier.buffer = device_data->indirectDispatchBuffer;
+            bufferBarrier.offset = 0;
+            bufferBarrier.size = VK_WHOLE_SIZE;
+            bufferBarrier.srcAccessMask = VK_ACCESS_SHADER_WRITE_BIT;
+            bufferBarrier.dstAccessMask = VK_ACCESS_INDIRECT_COMMAND_READ_BIT;
+            device_data->vtable.CmdPipelineBarrier(commandBuffer, VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT,
+                                                   VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, 0, 0, 1, &bufferBarrier, 0, 0);
         }
-    } catch (const std::bad_alloc& e) {
-        // We don't have a way to return an error here.
-        PRINT("bad_alloc: %s\n", e.what());
+
+        device_data->vtable.CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE, device_data->pipelineDecompressMulti);
+        device_data->vtable.CmdPushConstants(commandBuffer, device_data->pipelineLayoutDecompressMulti, VK_SHADER_STAGE_COMPUTE_BIT,
+                                             0, sizeof(DeviceData::PushConstantDataDecompressMulti), &pushConstantData);
+        device_data->vtable.CmdDispatchIndirect(commandBuffer, device_data->indirectDispatchBuffer, 0);
+        PRINT("Info: vkCmdDecompressMemoryIndirectCountNV: Using VK_LAYER_KHRONOS_memory_decompression layer\n");
     }
 }
 
@@ -901,6 +949,8 @@ static const std::unordered_map<std::string, PFN_vkVoidFunction> kInstanceFuncti
     ADD_HOOK(CreateDevice),
     ADD_HOOK(GetPhysicalDeviceProperties2),
     ADD_HOOK(GetPhysicalDeviceFeatures2),
+    ADD_HOOK(GetPhysicalDeviceProperties2KHR),
+    ADD_HOOK(GetPhysicalDeviceFeatures2KHR),
 };
 
 static const std::unordered_map<std::string, PFN_vkVoidFunction> kDeviceFunctions = {
