@@ -1,5 +1,5 @@
-/* Copyright (c) 2023-2024 LunarG, Inc.
- * Copyright (c) 2023-2024 Nintendo
+/* Copyright (c) 2023-2025 LunarG, Inc.
+ * Copyright (c) 2023-2025 Nintendo
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
  */
 
 #include <type_traits>
+#include <cmath>
 
 #include "extension_layer_tests.h"
 #include "shader_object_tests.h"
@@ -156,6 +157,9 @@ TEST_F(ShaderObjectTest, VertFragShader) {
     for (uint32_t i = 0; i < 2; ++i) {
         createInfos[i] = vku::InitStructHelper();
         createInfos[i].stage = shaderStages[i];
+        if (i == 0) {
+            createInfos[i].nextStage = VK_SHADER_STAGE_FRAGMENT_BIT;
+        }
         createInfos[i].codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
         createInfos[i].codeSize = spv[i].size() * sizeof(unsigned int);
         createInfos[i].pCode = spv[i].data();
@@ -272,9 +276,12 @@ TEST_F(ShaderObjectTest, VertFragShader) {
 
     float* data;
     vkMapMemory(m_device->handle(), buffer.memory().handle(), 0u, sizeof(float) * 4u, 0u, (void**)&data);
+    float e = 0.01f;
     for (uint32_t i = 0; i < 4; ++i) {
-        if (data[i] != 0.2f + i * 0.2f) {
-            m_errorMonitor->SetError("Wrong pixel value");
+        float expected = 0.2f + i * 0.2f;
+        if (std::fabs(data[i] - expected) > e) {
+            std::string msg = "Wrong pixel value " + std::to_string(data[i]) + ", expected " + std::to_string(0.2f + i * 0.2f);
+            m_errorMonitor->SetError(msg.c_str());
         }
     }
 
@@ -517,6 +524,9 @@ TEST_F(ShaderObjectTest, AllShadersDraw) {
     for (uint32_t i = 0; i < 5; ++i) {
         createInfos[i] = vku::InitStructHelper();
         createInfos[i].stage = shaderStages[i];
+        if (i < 4) {
+            createInfos[i].nextStage = shaderStages[i + 1];
+        }
         createInfos[i].codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
         createInfos[i].codeSize = spv[i].size() * sizeof(unsigned int);
         createInfos[i].pCode = spv[i].data();
@@ -696,6 +706,9 @@ TEST_F(ShaderObjectTest, AllShadersDrawBinary) {
     for (uint32_t i = 0; i < 5; ++i) {
         createInfos[i] = vku::InitStructHelper();
         createInfos[i].stage = shaderStages[i];
+        if (i < 4) {
+            createInfos[i].nextStage = shaderStages[i + 1];
+        }
         createInfos[i].codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
         createInfos[i].codeSize = spv[i].size() * sizeof(unsigned int);
         createInfos[i].pCode = spv[i].data();
@@ -715,6 +728,9 @@ TEST_F(ShaderObjectTest, AllShadersDrawBinary) {
 
         binaryCreateInfos[i] = vku::InitStructHelper();
         binaryCreateInfos[i].stage = shaderStages[i];
+        if (i < 4) {
+            binaryCreateInfos[i].nextStage = shaderStages[i + 1];
+        }
         binaryCreateInfos[i].codeType = VK_SHADER_CODE_TYPE_BINARY_EXT;
         binaryCreateInfos[i].codeSize = dataSize[i];
         binaryCreateInfos[i].pCode = binaryData[i].data();
@@ -992,6 +1008,9 @@ TEST_F(ShaderObjectTest, TaskMeshShadersDraw) {
     for (uint32_t i = 0; i < 3; ++i) {
         createInfos[i] = vku::InitStructHelper();
         createInfos[i].stage = shaderStages[i];
+        if (i < 2) {
+            createInfos[i].nextStage = shaderStages[i + 1];
+        }
         createInfos[i].codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
         createInfos[i].codeSize = spv[i].size() * sizeof(unsigned int);
         createInfos[i].pCode = spv[i].data();
@@ -1219,7 +1238,7 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
         GTEST_SKIP() << "VK_EXT_DYNAMIC_RENDERING_UNUSED_ATTACHMENTS_EXTENSION_NAME not supported";
     }
     m_device_extension_names.push_back(VK_EXT_DYNAMIC_RENDERING_UNUSED_ATTACHMENTS_EXTENSION_NAME);
-    if (!CheckShaderObjectSupportAndInitState()) {
+    if (!CheckShaderObjectSupportAndInitState(false)) {
         GTEST_SKIP() << kSkipPrefix << " shader object or dynamic rendering unused attachments not supported, skipping test";
     }
     if (DeviceValidationVersion() < VK_API_VERSION_1_1) {
@@ -1257,6 +1276,9 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
     for (uint32_t i = 0; i < 2; ++i) {
         createInfos[i] = vku::InitStructHelper();
         createInfos[i].stage = shaderStages[i];
+        if (i == 0) {
+            createInfos[i].nextStage = shaderStages[i + 1];
+        }
         createInfos[i].codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
         createInfos[i].codeSize = spv[i].size() * sizeof(unsigned int);
         createInfos[i].pCode = spv[i].data();
@@ -1355,9 +1377,9 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
         vkCmdBindShadersEXT(m_commandBuffer->handle(), 1u, &unusedShader, &null_shader);
     }
     BindDefaultDynamicStates(vertexBuffer.handle(), false);
-    {
+    for (uint32_t i = 0; i < 3; ++i) {
         VkBool32 colorBlendEnable = VK_FALSE;
-        vkCmdSetColorBlendEnableEXT(m_commandBuffer->handle(), 1u, 1u, &colorBlendEnable);
+        vkCmdSetColorBlendEnableEXT(m_commandBuffer->handle(), i, 1u, &colorBlendEnable);
         VkColorBlendEquationEXT colorBlendEquation = {
             VK_BLEND_FACTOR_ONE,  // VkBlendFactor	srcColorBlendFactor;
             VK_BLEND_FACTOR_ONE,  // VkBlendFactor	dstColorBlendFactor;
@@ -1366,10 +1388,10 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
             VK_BLEND_FACTOR_ONE,  // VkBlendFactor	dstAlphaBlendFactor;
             VK_BLEND_OP_ADD,      // VkBlendOp		alphaBlendOp;
         };
-        vkCmdSetColorBlendEquationEXT(m_commandBuffer->handle(), 1u, 1u, &colorBlendEquation);
+        vkCmdSetColorBlendEquationEXT(m_commandBuffer->handle(), i, 1u, &colorBlendEquation);
         VkColorComponentFlags colorWriteMask =
             VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
-        vkCmdSetColorWriteMaskEXT(m_commandBuffer->handle(), 1u, 1u, &colorWriteMask);
+        vkCmdSetColorWriteMaskEXT(m_commandBuffer->handle(), i, 1u, &colorWriteMask);
     }
     vkCmdDraw(m_commandBuffer->handle(), 4, 1, 0, 0);
     vkCmdEndRenderingKHR(m_commandBuffer->handle());
@@ -1385,6 +1407,7 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
     vkCmdEndRenderingKHR(m_commandBuffer->handle());
 
     begin_rendering_info.colorAttachmentCount = 3u;
+    color_attachments[1].imageLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
     color_attachments[1].imageView = view2;
     vkCmdBeginRenderingKHR(m_commandBuffer->handle(), &begin_rendering_info);
     vkCmdDraw(m_commandBuffer->handle(), 4, 1, 0, 0);
@@ -1438,14 +1461,19 @@ TEST_F(ShaderObjectTest, UnusedAttachments) {
     vkMapMemory(m_device->handle(), buffer1.memory().handle(), 0u, sizeof(float) * 4u, 0u, (void**)&data1);
     vkMapMemory(m_device->handle(), buffer2.memory().handle(), 0u, sizeof(float) * 4u, 0u, (void**)&data2);
 
+    float e = 0.01f;
     for (uint32_t i = 0; i < 4; ++i) {
-        if (data1[i] != 0.2f + i * 0.2f) {
-            m_errorMonitor->SetError("Wrong pixel value");
+        float expected = 0.2f + i * 0.2f;
+        if (std::fabs(data1[i] - expected) > e) {
+            std::string msg = "Wrong pixel value " + std::to_string(data1[i]) + ", expected " + std::to_string(0.2f + i * 0.2f);
+            m_errorMonitor->SetError(msg.c_str());
         }
     }
     for (uint32_t i = 0; i < 4; ++i) {
-        if (data2[i] != 0.2f + i * 0.2f) {
-            m_errorMonitor->SetError("Wrong pixel value");
+        float expected = 0.2f + i * 0.2f;
+        if (std::fabs(data2[i] - expected) > e) {
+            std::string msg = "Wrong pixel value " + std::to_string(data2[i]) + ", expected " + std::to_string(0.2f + i * 0.2f);
+            m_errorMonitor->SetError(msg.c_str());
         }
     }
 
